/*
 * Copyright (c) 2007-2008, Opera Software ASA
 * All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of Opera Software ASA nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY OPERA SOFTWARE ASA AND CONTRIBUTORS ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL OPERA SOFTWARE ASA AND CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

var ctx, old_s_a = 0;

var config = {
      SKINS         : [
                    "white"
                    ,"black"
                    ,"metal"
                    ,"gold"
                    ]
      ,SMOOTH       : false
      ,TIMEOUT      : 72
      ,PADDING      : 0
      ,GRAPHICAL    : false
      ,WIDTH        : 256
      ,HEIGHT       : 256
      ,OFFSET       : -16
      ,HOUR_LENGTH  : 0
      ,HOUR_WIDTH   : 0
      ,HOUR_OFFSET  : 0
      ,HOUR_IMAGE   : new Image()
      ,HOUR_COLOR   : null
      ,MIN_LENGTH   : 0
      ,MIN_WIDTH    : 0
      ,MIN_OFFSET   : 0
      ,MIN_IMAGE    : new Image()
      ,MIN_COLOR    : null
      ,SEC_LENGTH   : 0
      ,SEC_WIDTH    : 0
      ,SEC_OFFSET   : 0
      ,SEC_IMAGE    : new Image()
      ,SEC_COLOR    : null
};

window.addEventListener("load", function(ev)
{
    if (navigator.platform.toLowerCase().indexOf('linux') > -1 )
    {
        document.body.className += " linux";
    }

    ctx = document.getElementById("playfield").getContext("2d");
    
    document.getElementById('mainview').className = config.SKINS[skinCounter];
    
    configure();
    clock();
}
,false);

function configure()
{
    var hour_css = window.getComputedStyle(document.getElementById("hours"),null),
        min_css = window.getComputedStyle(document.getElementById("minutes"),null),
        sec_css = window.getComputedStyle(document.getElementById("seconds"),null);

    config.HAS_SHADOW   = (hour_css.getPropertyValue("padding-top") != "none")?true:false;
    config.HOUR_LENGTH  = parseInt(hour_css.getPropertyValue("height"), 10);
    config.HOUR_WIDTH   = parseInt(hour_css.getPropertyValue("width"), 10);
    config.HOUR_OFFSET  = h_off = -parseInt(hour_css.getPropertyValue("margin-top"), 10);

    config.MIN_LENGTH   = parseInt(min_css.getPropertyValue("height"), 10);
    config.MIN_WIDTH    = parseInt(min_css.getPropertyValue("width"), 10);
    config.MIN_OFFSET   = -parseInt(min_css.getPropertyValue("margin-top"), 10);

    config.SEC_LENGTH   = parseInt(sec_css.getPropertyValue("height"), 10);
    config.SEC_WIDTH    = parseInt(sec_css.getPropertyValue("width"), 10);
    config.SEC_OFFSET   = -parseInt(sec_css.getPropertyValue("margin-top"), 10);

    config.GRAPHICAL = hour_css.getPropertyValue("content") !== "none";
    if (!config.GRAPHICAL)
    {
        config.HOUR_COLOR = hour_css.getPropertyValue("background-color");
        config.MIN_COLOR  = min_css.getPropertyValue("background-color");
        config.SEC_COLOR  = sec_css.getPropertyValue("background-color");
    }
    else
    {
        config.HOUR_IMAGE.src = hour_css.getPropertyValue("content");
        config.MIN_IMAGE.src  = min_css.getPropertyValue("content");
        config.SEC_IMAGE.src  = sec_css.getPropertyValue("content");
    }
}

function drawShadow(offset, hand, angle)
{
    ctx.save();
    ctx.translate(config.WIDTH/2, config.HEIGHT/2);
    ctx.translate(0,offset);
    ctx.rotate(angle);
    ctx.translate(-(config[hand+"_WIDTH"]/2), config[hand+"_OFFSET"]);
    ctx.fillStyle = "rgba(0,0,0,0.25)";
    ctx.fillRect(0,0,config[hand+"_WIDTH"]+1, config[hand+"_LENGTH"]);
    ctx.restore();
}

function drawHand(hand, angle)
{
    ctx.save();
    ctx.translate(config.WIDTH/2,config.HEIGHT/2);
    ctx.rotate(angle);
    ctx.translate(-(config[hand+"_WIDTH"]/2), config[hand+"_OFFSET"]);
    if (!config.GRAPHICAL)
    {
        ctx.fillStyle = config[hand+"_COLOR"];
        ctx.fillRect(0,0,config[hand+"_WIDTH"], config[hand+"_LENGTH"]);
    }
    else
    {
        ctx.drawImage(config[hand+"_IMAGE"]);
    }
    ctx.restore();
}


function clock()
{
    var now      = new Date(),
        hours    = now.getHours() % 12,
        minutes  = now.getMinutes(),
        seconds  = now.getSeconds(),
        mseconds = config.SMOOTH ? now.getMilliseconds() : 0;
    
    dock.textContent = zeroPad(now.getHours(), 2) + ":" + zeroPad(minutes, 2) + ":" + zeroPad(seconds, 2);

    var h_a = (hours/6 + minutes/360 + seconds/21600) * Math.PI + Math.PI;
    var m_a = (minutes/30 + seconds/1800) * Math.PI + Math.PI;
    var s_a = (seconds/30 + mseconds/30000) * Math.PI + Math.PI;
    if (old_s_a != s_a)
    {
        document.getElementById("date").textContent = now.getDate();
        ctx.globalCompositeOperation = "copy";
        // Draw Hours
        if (config.HAS_SHADOW)
        {
            drawShadow(1, "HOUR", h_a);
        }
        ctx.globalCompositeOperation = "source-over";
        drawHand("HOUR", h_a)

        // Minutes
        if (config.HAS_SHADOW)
        {
            drawShadow(2, "MIN", m_a);
        }
        drawHand("MIN",m_a);
        // Seconds
        if (config.HAS_SHADOW)
        {  
            drawShadow(3, "SEC", s_a);
        }
        drawHand("SEC",s_a);
    }
    old_s_a = s_a;
    setTimeout(clock, config.TIMEOUT);
}

var skinCounter = widget.preferences.skinCounter || 0,
    oldEvent;

//window.addEventListener("click", setSkin, false);
window.addEventListener("mousedown", function(event) {
    oldEvent = event;
}, false);

window.addEventListener("mouseup", function(event) {
    if(oldEvent) {
        var dx = oldEvent.screenX - event.screenX,
            dy = oldEvent.screenY - event.screenY;
        
        if(Math.sqrt(dx*dx + dy*dy) < 5) {
            skinCounter = (skinCounter + 1) % config.SKINS.length;
            setSkin(skinCounter);
        }

        oldEvent = null;
    }
}, false);

function setSkin(counter) {
    document.getElementById('mainview').className = config.SKINS[counter];
    
    configure();
    widget.preferences.setItem(counter, 'skinCounter');
}

function zeroPad(value, totalLength)
{
    return ("00000000000000" + value).slice(- totalLength)
}
